%%%%    This program is developed by Dr. Hamed Ahmadi to calculate the expected 
%%%%    number of simultaneous outages of two parallel transmission lines. 
%%%%    Copyright (C) 2019  Hamed Ahmadi
%%%%
%%%%    This program is free software: you can redistribute it and/or modify
%%%%    it under the terms of the GNU General Public License as published by
%%%%    the Free Software Foundation, either version 3 of the License, or
%%%%    any later version.
%%%%
%%%%    This program is distributed in the hope that it will be useful,
%%%%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%%%%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%%%%    GNU General Public License for more details.
%%%%
%%%%    For a copy of the GNU General Public License
%%%%    see <https://www.gnu.org/licenses/>.
%%%%    
clc
clear
close all;
x = [15;20;23;31;12;18;22;28;13;29];%Present distance between closest matallic groundings
rho = [1000;800;1600;3500;500;700;200;600;1700;2300];%soil resistivity (ohm.m) for each strucure
Rg = [200;162;314;190;92;65;30;55;229;181];%Low frequency tower grounding resistance
N_Split = [2;2;2;4;2;4;2;4;2;4];%Number of structure footings amongst which the current will split
D_foundation = [0.4;0.4;0.4;3.0;0.4;3.0;0.4;2.9;0.5;2.9];%Half the size of the foundation (m)
L_insulator = 1.75;%Length of the insulator string (m)
t_cr = 6;%Critical time to flashover (us)
V_W = (400+710/t_cr^0.75)*L_insulator;%Lightning Withstand Voltage of the insulators of the second circuit (kV) 
Ng = 1;%Ground flash density (flashes/km^2/year)
d1= dlmread('Str_Cond_Flashes.txt');%SESShield3D output for conductors
%%%Lightning Current (kA)	    No. Flashes/Span/Year	      No. Flashes/Str/Year

Nc = Ng*d1(:,2);%Flashes collected by each span by all the conductors 
Nt = Ng*d1(:,3);%Flashes collected by each structure 
I = d1(:,1);%representative current magnitude for each interval

N_I = length(I);%Number of intervals
N_T = length(x);%Number of towers
E0 = 3e+5;%Ionization gradient of soil near tower footing
Eb = 0.5e+5;%Average gradient across the gap that causes complete breakdown of soil

for j = 1:N_T
    for k = 1:N_I
%%%%%%%for flashes terminated on conductors
    Ig = I(k)/2/N_Split(j);%Current entering the ground through each leg
    r_hem = sqrt(rho(j)*1000*Ig/(2*pi*E0));%Radius of the hemisphere formed at each leg
    R_hem = rho(j)./(2*pi*r_hem);%Resistance of ionized zone;%grounding resistance of a hemisphere formed at each leg, collision of hemispheres is ignored
    %%%%If the low-freq grounding resistance is smaller than the ionized zone
    if R_hem/N_Split(j)>Rg(j)
       R_eq = Rg(j)*N_Split(j);
    else
       R_eq = R_hem;
    end
    %%If the foundation sticks outside the ionized zone
    if D_foundation(j)>r_hem
       r_eq = D_foundation(j);
    else
       r_eq = r_hem;
    end   
    GPR = 1000*R_eq.*Ig;%Ground potential rise (V)
    Xmin = r_eq + GPR/Eb;% Minimum separation to cause an arc (m)
      if x(j)<=Xmin && GPR > V_W*1000
          NC(j,k) = Nc(k);%Number or arc events caused by the two neighboring spans
      else
          NC(j,k) = 0;
      end
%%%%%%%for flashes terminated on structures
    Ig = I(k)/N_Split(j);%Current entering the ground through each leg
    r_hem = sqrt(rho(j)*1000*Ig/(2*pi*E0));%Radius of the hemisphere formed at each leg
    R_hem = rho(j)./(2*pi*r_hem);%Resistance of ionized zone;%grounding resistance of a hemisphere formed at each leg, collision of hemispheres is ignored
    %%%%If the low-freq grounding resistance is smaller than the ionized zone
    if R_hem/N_Split(j)>Rg(j)
       R_eq = Rg(j)*N_Split(j);
    else
       R_eq = R_hem;
    end
    %%If the foundation sticks outside the ionized zone
    if D_foundation(j)>r_hem
       r_eq = D_foundation(j);
    else
       r_eq = r_hem;
    end   
    GPR = 1000*R_eq.*Ig;%Ground potential rise (V)
    Xmin = r_eq + GPR/Eb;% Minimum separation to cause an arc (m)
      if x(j)<=Xmin && GPR > V_W*1000
          NT(j,k) = Nt(k);%Number or arc events caused by the two neighboring spans
      else
          NT(j,k) = 0;
      end 
    end
    N_Total_T(j) = sum(NT(j,:));
    N_Total_C(j) = sum(NC(j,:));
end

N_All = sum(N_Total_T + N_Total_C)
subplot(3,1,1);bar(N_Total_T);title("Strikes to Towers");ylabel("Number of Outages/Year")
subplot(3,1,2);bar(N_Total_C);title("Strikes to Conductors");ylabel("Number of Outages/Year")
subplot(3,1,3);bar(N_Total_T + N_Total_C);title("Total");ylabel("Number of Outages/Year")

display("Total Number of Outages per Structure:")
display([N_Total_T + N_Total_C]')
